@@ -1,13 +1,15 @@
 from __future__ import unicode_literals
 
 import logging
-import pykka
 
 from mopidy import backend
 
+import pykka
+
 from .library import SoundCloudLibraryProvider
 from .soundcloud import SoundCloudClient
 
+
 logger = logging.getLogger(__name__)
 
 
@@ -25,7 +27,10 @@ class SoundCloudBackend(pykka.ThreadingActor, backend.Backend):
 
 class SoundCloudPlaybackProvider(backend.PlaybackProvider):
 
-    def play(self, track):
-        track_id = self.backend.remote.parse_track_uri(track)
+    def translate_uri(self, uri):
+        track_id = self.backend.remote.parse_track_uri(uri)
         track = self.backend.remote.get_track(track_id, True)
-        return super(SoundCloudPlaybackProvider, self).play(track)
+        if hasattr(track, 'uri'):
+            return track.uri
+        else:
+            return None
