@@ -1,38 +1,33 @@
 from __future__ import unicode_literals
+
+import collections
 import logging
 import re
 import string
 import time
-from urllib import quote_plus
-import collections
 import unicodedata
+from multiprocessing.pool import ThreadPool
+from urllib import quote_plus
+
+from mopidy.models import Album, Artist, Track
 
 import requests
 
-from mopidy.models import Track, Artist, Album
-
 
 logger = logging.getLogger(__name__)
 
 
 def safe_url(uri):
-    return quote_plus(unicodedata.normalize(
-        'NFKD',
-        unicode(uri)
-    ).encode('ASCII', 'ignore'))
+    return quote_plus(
+        unicodedata.normalize('NFKD', unicode(uri)).encode('ASCII', 'ignore'))
 
 
 def readable_url(uri):
     valid_chars = "-_.() %s%s" % (string.ascii_letters, string.digits)
-    safe_uri = unicodedata.normalize(
-        'NFKD',
-        unicode(uri)
-    ).encode('ASCII', 'ignore')
-    return re.sub(
-        '\s+',
-        ' ',
-        ''.join(c for c in safe_uri if c in valid_chars)
-    ).strip()
+    safe_uri = unicodedata.normalize('NFKD', unicode(uri)).encode('ASCII',
+                                                                  'ignore')
+    return re.sub('\s+', ' ',
+                  ''.join(c for c in safe_uri if c in valid_chars)).strip()
 
 
 class cache(object):
@@ -75,10 +70,19 @@ class SoundCloudClient(object):
     def __init__(self, config):
         super(SoundCloudClient, self).__init__()
         token = config['auth_token']
-        self.explore_songs = config['explore_songs']
+        self.explore_songs = config.get('explore_songs', 10)
         self.http_client = requests.Session()
         self.http_client.headers.update({'Authorization': 'OAuth %s' % token})
 
+        try:
+            self._get('me.json')
+        except Exception as err:
+            if err.response.status_code == 401:
+                logger.error('Invalid "auth_token" used for SoundCloud '
+                             'authentication!')
+            else:
+                raise
+
     @property
     @cache()
     def user(self):
@@ -100,31 +104,33 @@ class SoundCloudClient(object):
                 if kind == 'playlist':
                     playlist = data.get('playlist').get('tracks')
                     if isinstance(playlist, collections.Iterable):
-                        tracks.extend(self.parse_results())
+                        tracks.extend(self.parse_results(playlist))
 
         return self.sanitize_tracks(tracks)
 
-    def get_explore(self, query_explore_id=None):
-        explore = self._get('explore/categories', 'api-v2').get('music')
-        if query_explore_id:
-            urn = explore[int(query_explore_id)]
-            web_tracks = self._get(
-                'explore/%s?tag=%s&limit=%s&offset=0&linked_partitioning=1' %
-                (urn, 'out-of-experiment', self.explore_songs),
-                'api-v2'
-            )
-            tracks = []
-            for track in web_tracks.get('tracks'):
-                tracks.append(self.get_track(track.get('id')))
-            return tracks
+    @cache()
+    def get_explore_categories(self):
+        return self._get('explore/categories', 'api-v2').get('music')
 
+    def get_explore(self, query_explore_id=None):
+        explore = self.get_explore_categories()
+        if query_explore_id:
+
+            url = 'explore/{urn}?limit={limit}&offset=0&linked_partitioning=1'\
+                .format(
+                    urn=explore[int(query_explore_id)],
+                    limit=self.explore_songs
+                )
+            web_tracks = self._get(url, 'api-v2')
+            track_ids = map(lambda x: x.get('id'), web_tracks.get('tracks'))
+            return self.resolve_tracks(track_ids)
         return explore
 
     def get_groups(self, query_group_id=None):
 
         if query_group_id:
-            web_tracks = self._get('groups/%d/tracks.json' % int(
-                query_group_id))
+            web_tracks = self._get(
+                'groups/%d/tracks.json' % int(query_group_id))
             tracks = []
             for track in web_tracks:
                 if 'track' in track.get('kind'):
@@ -139,14 +145,12 @@ class SoundCloudClient(object):
             return self._get('users/%s/tracks.json' % query_user_id)
 
         users = []
-        for playlist in self._get('me/followings.json?limit=1000'):
+        for playlist in self._get('me/followings.json?limit=60'):
             name = playlist.get('username')
             user_id = str(playlist.get('id'))
-            logger.debug(
-                'Fetched user %s with id %s' % (
-                    name, user_id
-                )
-            )
+            logger.debug('Fetched user %s with id %s' % (
+                name, user_id
+            ))
 
             users.append((name, user_id))
         return users
@@ -162,11 +166,9 @@ class SoundCloudClient(object):
             name = playlist.get('title')
             set_id = str(playlist.get('id'))
             tracks = playlist.get('tracks')
-            logger.debug(
-                'Fetched set %s with id %s (%d tracks)' % (
-                    name, set_id, len(tracks)
-                )
-            )
+            logger.debug('Fetched set %s with id %s (%d tracks)' % (
+                name, set_id, len(tracks)
+            ))
             playable_sets.append((name, set_id, tracks))
         return playable_sets
 
@@ -191,10 +193,8 @@ class SoundCloudClient(object):
     def get_track(self, track_id, streamable=False):
         logger.debug('Getting info for track with id %s' % track_id)
         try:
-            return self.parse_track(
-                self._get('tracks/%s.json' % track_id),
-                streamable
-            )
+            return self.parse_track(self._get('tracks/%s.json' % track_id),
+                                    streamable)
         except Exception:
             logger.info('Song %s was removed' % track_id)
             return Track()
@@ -208,9 +208,8 @@ class SoundCloudClient(object):
     def search(self, query):
 
         search_results = self._get(
-            'tracks.json?q=%s&filter=streamable&order=hotness&limit=10' %
-            quote_plus(query)
-        )
+            'tracks.json?q=%s&filter=streamable&order=hotness&limit=%d' % (
+                quote_plus(query), self.explore_songs))
         tracks = []
         for track in search_results:
             tracks.append(self.parse_track(track, False))
@@ -247,8 +246,7 @@ class SoundCloudClient(object):
             return []
         if not data['streamable']:
             logger.info(
-                "'%s' can't be streamed from SoundCloud" % data.get('title')
-            )
+                "'%s' can't be streamed from SoundCloud" % data.get('title'))
             return []
         if not data['kind'] == 'track':
             logger.debug('%s is not track' % data.get('title'))
@@ -280,8 +278,8 @@ class SoundCloudClient(object):
         if remote_url:
             if not self.can_be_streamed(data['stream_url']):
                 logger.info(
-                    "'%s' can't be streamed from SoundCloud" %
-                    data.get('title'))
+                    "'%s' can't be streamed from SoundCloud" % data.get(
+                        'title'))
                 return []
             track_kwargs[b'uri'] = self.get_streamble_url(data['stream_url'])
         else:
@@ -316,3 +314,17 @@ class SoundCloudClient(object):
 
     def get_streamble_url(self, url):
         return '%s?client_id=%s' % (url, self.CLIENT_ID)
+
+    def resolve_tracks(self, track_ids):
+        """Resolve tracks concurrently emulating browser
+
+        :param track_ids:list of track ids
+        :return:list `Track`
+        """
+        pool = ThreadPool(processes=16)
+        tracks = pool.map(self.resolve_track, track_ids)
+        pool.close()
+        return tracks
+
+    def resolve_track(self, track_id):
+        return self.get_track(track_id)
